/*
 * UseCaseXML.java
 *
 * Created on 15 de abril de 2006, 13:14
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.usecase;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

// XML imports
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 *
 * @author Rincew
 */
public class UseCaseXML 
{
    
    protected Document useCaseDocument;
    protected StepBuilder stepBuilder;
    
    //--- Iterators attributes

    
    /** Stores the list of nodes in the alternative
     * sequence (tags with "alstep" name) */
    protected NodeList alternativeSequence;
    protected int alternativeSequenceIndex;
        
    /** Creates a new instance of UseCaseXML */
    public UseCaseXML () {
        this.useCaseDocument = null;
        this.stepBuilder = new StepBuilder();
    }
    
    /** 
     * Loads a XML file with the definition of a use case 
     * This method assumes that file exists.
     **/
    public void load(File f) 
        throws UseCaseXMLException
    {
        DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance ();
        DocumentBuilder dom;
        try {
            dom = domFactory.newDocumentBuilder();
            this.useCaseDocument = dom.parse (f);
        } catch (SAXException ex) {
            UseCaseXMLException exc = new UseCaseXMLException( "XML error " 
                + " when reading " + f +". \n" + ex.toString ()    );
            exc.setStackTrace (ex.getStackTrace ());
            throw exc;
            
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
       
    }
    
    /**
     * Returns an iterator to traverse all the steps in the main sequence of
     *the use case
     **/
    public Iterator iteratorMainSequence() {
        return new IteratorMainSequence(this.useCaseDocument.getElementsByTagName ("step"));
    }

    /**
     * Returns an iterator to traverse all the pre steps in the main 
     * sequence of the use case
     **/
    public Iterator iteratorPreSequence() {
        return new IteratorPreSequence(this.useCaseDocument.getElementsByTagName ("alstep"));
    }
    
    /** Testing **/
    public String toString() {
        StringBuffer sb = new StringBuffer("");
        Iterator it = this.iteratorMainSequence ();
        
        while (it.hasNext ())
            sb.append (it.next ()+"\n");
        
        sb.append ("Pre alternatives;\n");
        
        it = this.iteratorPreSequence ();
        while (it.hasNext ())
            sb.append (it.next ()+"\n");
        
        return sb.toString ();
    }
    
    
    //--- Iterator classes -------------------------------
    
    private class IteratorMainSequence 
            implements Iterator
    {
        /** Stores the list of nodes in the main
         * sequence (tags with "step" name) */
        protected NodeList mainSequence;
        protected int mainSequenceIndex;

        public IteratorMainSequence(NodeList nodes) {
                this.mainSequenceIndex = 0;
                this.mainSequence = nodes;
        }
    
        public boolean hasNext () {
            return (this.mainSequenceIndex < this.mainSequence.getLength ());
        }

        public Object next () {
           Node n; 
           n = this.mainSequence.item (this.mainSequenceIndex++);
        
            return stepBuilder.builMainStep (n);
        }

        /** Operation not implemented */
        public void remove () {
            throw new java.lang.UnsupportedOperationException("Not impleented");
        }

    } // Class IteratorMainSequence

    /** **/
    private class IteratorPreSequence 
            implements Iterator
    {
        /** Stores the list of nodes in the main
         * sequence (tags with "step" name) */
        protected NodeList preSequence;
        protected int preSequenceIndex;

        public IteratorPreSequence(NodeList nodes) {
                this.preSequenceIndex = 0;
                this.preSequence = nodes;
        }
    
        public boolean hasNext () {
           Node n, a; 
           int i = 1;
           int tmpIndex = this.preSequenceIndex;
           
           if ( this.preSequence.getLength () == 0 )
               return false;
           if ( tmpIndex >= this.preSequence.getLength ())
               return false;
  
           do {
                n = this.preSequence.item (tmpIndex++);
                /*
                if (n == null)
                    System.err.println ("El nodo es nulo; "+i+" / "+tmpIndex + " / " + this.preSequence.getLength ());
                
                a = n.getAttributes ().getNamedItem ("type");
                if (a == null)
                    System.err.println ("El atributo es nulo; "+i);
                
                i++;
                */
           } while ( (n.getAttributes ().getNamedItem ("type").getNodeValue ().compareTo ("pre")!= 0) &&
                   (tmpIndex <= (this.preSequence.getLength ()) ) ) ;
             
           
            // System.err.println ("Salida; "+i+" / "+tmpIndex + " / " + this.preSequence.getLength ());
            return ( (tmpIndex-1) < this.preSequence.getLength ());
        }

        /** **/
        public Object next () {
           Node n; 
           n = this.preSequence.item (this.preSequenceIndex++);
           
           while ( (n.getAttributes ().getNamedItem ("type").getNodeValue ().compareTo ("pre")!= 0) && 
                   (this.hasNext ()) )
               n = this.preSequence.item (this.preSequenceIndex++);
           
           return stepBuilder.builMainStep (n);
        }

        /** Operation not implemented */
        public void remove () {
            throw new java.lang.UnsupportedOperationException("Not impleented");
        }

    } // Class IteratorPreSequence

}
